#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from __future__ import unicode_literals

import os
import sys

import pytest
from odictliteral import odict

import cz_urnnbn_api
from cz_urnnbn_api.xml_composer import MonographComposer
from cz_urnnbn_api.xml_composer import MultiMonoComposer


# Functions & classes =========================================================
def get_data_path():
    wd = os.path.dirname(__file__)
    return os.path.join(wd, "data")


def data_context(fn):
    path = os.path.join(get_data_path(), fn)
    with open(path) as f:
        return f.read()


# Fixtures ====================================================================
@pytest.fixture
def mono_out_example():
    return data_context("mono_output.xml")


@pytest.fixture
def multimono_out_example():
    return data_context("mono_volume_output.xml")


# Tests =======================================================================
def test_mono_xml_conversion(mono_out_example):
    out = MonographComposer(
        title="Dračí doupě plus",
        ccnb="cnb001852175",
        isbn="978-80-85979-67-1",
        other_id="87d27370-db5b-11e3-b110-005054787e51",
        document_type="elektronický zdroj",
        digital_born="true",
        author="Ježek, Matouš",
        publisher="Altar",
        place="Ostrava",
        year="2012",
        format="pdf",
    )

    assert out.to_xml() == mono_out_example


def test_mono_xml_without_uuid(mono_out_example):
    out = MonographComposer(
        title="Dračí doupě plus",
        ccnb="cnb001852175",
        isbn="978-80-85979-67-1",
        document_type="elektronický zdroj",
        digital_born="true",
        author="Ježek, Matouš",
        publisher="Altar",
        place="Ostrava",
        year="2012",
        format="pdf",
    )

    # remove uuid
    mono = mono_out_example.decode("utf-8").splitlines()
    del mono[8]
    mono = "\n".join(mono)

    assert out.to_xml().decode("utf-8") == mono


def test_mono_xml_without_invalid_parameter():
    with pytest.raises(ValueError):
        MonographComposer(
            title="Dračí doupě plus",
            ccnb="cnb001852175",
            isbn="978-80-85979-67-1",
            azgabash="elektronický zdroj"
        )


def test_swap_keys():
    original = odict[
        "somekey": "azgabash",
        "oldkey": "something",
        "another_key": "asd"
    ]

    new_dict = MultiMonoComposer._swap_keys(original, "oldkey", "newkey")

    assert new_dict == odict[
        "somekey": "azgabash",
        "newkey": "something",
        "another_key": "asd"
    ]


def test_mono_volume_xml_conversion(multimono_out_example):
    out = MultiMonoComposer(
        title="Dračí doupě plus",
        ccnb="cnb001852175",
        isbn="978-80-85979-67-1",
        other_id="87d27370-db5b-11e3-b110-005054787e51",
        document_type="elektronický zdroj",
        digital_born="true",
        author="Ježek, Matouš",
        publisher="Altar",
        place="Ostrava",
        year="2012",
        format="pdf",
        volume_title="1",
    )

    assert out.to_xml() == multimono_out_example


def test_create_path():
    r = odict(key="hello")

    MonographComposer._create_path(r, odict, ["subpath", "more"])

    assert r == odict[
        "key": "hello",
        "subpath": odict[
            "more": odict()
        ]
    ]


def test_create_path_existing_keys():
    r = odict(key="hello")

    more = MonographComposer._create_path(r, odict, ["key", "subpath", "more"])

    assert r == odict[
        "key": odict[
            "subpath": odict[
                "more": odict()
            ]
        ]
    ]

    more["asd"] = "val"
    assert more["asd"] == r["key"]["subpath"]["more"]["asd"]


def test_assign_pattern():
    r = odict(key="hello")

    MonographComposer._assign_pattern(r, "otherkey", False)

    assert r == odict(key="hello")

    MonographComposer._assign_pattern(r, "otherkey", "hi")

    assert r == odict[
        "key": "hello",
        "otherkey": "hi"
    ]


def test_setting_uset_is_disabled():
    mc = MonographComposer()
    mmc = MultiMonoComposer()

    with pytest.raises(ValueError):
        mc.azgabash = True

    with pytest.raises(ValueError):
        mmc.azgabash = True



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================


# Functions & classes =========================================================
class URN_NBN(str):
    """
    Class used to hold URN:NBN string and also other informations returned
    from server.

    Note:
        This class subclasses ``str``, so URN:NBN string can be obtained
        painlessly.

    Attributes:
        value (str): Whole URN:NBN string.
        status (str): ``ACTIVE`` for example.
        registered (str): ISO 8601 date string.
        country_code (str): Code of the country (``cz`` for URN:NBN).
        document_code (str): Part of the URN:NBN holding the code.
        registrar_code (str): Identification of registrar.
        digital_document_id (str): ID of the document.
    """
    def __new__(self, value, *args, **kwargs):
        return super(URN_NBN, self).__new__(self, value)

    def __init__(self, value, status=None, country_code=None,
                 registrar_code=None, document_code=None,
                 digital_document_id=None, registered=None):
        self.value = value

        super(URN_NBN, self).__init__(value)

        self.status = status
        self.registered = registered
        self.country_code = country_code
        self.document_code = document_code
        self.registrar_code = registrar_code
        self.digital_document_id = digital_document_id

    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value

    @staticmethod
    def from_xmldict(xdom, tag_picker=lambda x: x["response"]["urnNbn"]):
        """
        Parse itself from `xmldict` structure.
        """
        urn_nbn_tag = tag_picker(xdom)

        return URN_NBN(
            value=urn_nbn_tag["value"],
            status=urn_nbn_tag["status"],
            registered=urn_nbn_tag.get("registered", None),
            country_code=urn_nbn_tag.get("countryCode", None),
            document_code=urn_nbn_tag.get("documentCode", None),
            registrar_code=urn_nbn_tag.get("registrarCode", None),
            digital_document_id=urn_nbn_tag.get("digitalDocumentId", None),
        )



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from tools import both_set_and_different


# Functions & classes =========================================================
class DigitalLibrary(object):
    """
    Container used to hold informations about given digital library, where the
    document to which the URN:NBN points, stored.

    Attributes:
        url (str): URL of the library.
        uid (str): ID of the library.
        name (str): Name of the digital library.
        created (str): ISO 8601 string.
        description (str): Free text description of the library.
    """
    def __init__(self, uid, name, description=None, url=None, created=None):
        self.url = url
        self.uid = uid
        self.name = name
        self.created = created
        self.description = description

    def __eq__(self, other):
        if self.uid != other.uid or self.name != other.name:
            return False

        return not any([
            both_set_and_different(self.description, other.description),
            both_set_and_different(self.url, other.url),
            both_set_and_different(self.created, other.created),
        ])

    def __ne__(self, other):
        return not self.__eq__(other)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import pytest

from cz_urnnbn_api.api_structures import Modes


# Tests =======================================================================
def test_modes_creation():
    m = Modes(
        by_resolver=True,
        by_registrar=True,
        by_reservation=True,
    )

    assert m.by_resolver
    assert m.by_registrar
    assert m.by_reservation


    m = Modes(
        by_resolver=False,
        by_registrar=True,
        by_reservation=False,
    )

    assert not m.by_resolver
    assert m.by_registrar
    assert not m.by_reservation


def test_modes_op_equal():
    m = Modes(
        by_resolver=True,
        by_registrar=True,
        by_reservation=True,
    )

    m2 = Modes(
        by_resolver=False,
        by_registrar=True,
        by_reservation=False,
    )

    assert not m == m2
    assert m != m2


def test_modes_op_equal_same_params():
    m = Modes(
        by_resolver=True,
        by_registrar=True,
        by_reservation=True,
    )

    m2 = Modes(
        by_resolver=True,
        by_registrar=True,
        by_reservation=True,
    )

    assert m == m2
    assert not m != m2



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import xmltodict
from odictliteral import odict
from kwargs_obj import KwargsObj

from tools import both_set_and_different


# Functions & classes =========================================================
class DigitalInstance(KwargsObj):
    """
    Container used to hold informations about instances of the documents in
    digital library - this is pointer to document in digital library.

    Attributes:
        uid (str): ID of the library.
        url (str): URL of the library.
        digital_library_id (str): Id of the digitial library.
        active (bool, def. None): Is the record active?
        created (str, def. None): ISO 8601 string with date.
        deactivated (str, def. None): ISO 8601 string representation of date.
        format (str, def. None): Format of the book. ``jpg;pdf`` for example.
        accessibility (str, def. None): Free description of accessibility.
    """
    def __init__(self, url, digital_library_id, **kwargs):
        self.url = url
        self.digital_library_id = digital_library_id
        self.uid = None
        self.active = None
        self.format = None
        self.created = None
        self.accessibility = None
        self.deactivated = None

        self._all_set = True
        self._kwargs_to_attributes(kwargs)

    def __eq__(self, other):
        tests = [
            self.uid != other.uid,
            self.active != other.active,
            self.digital_library_id != other.digital_library_id,
            self.url != other.url,
            self.deactivated != other.deactivated,
        ]
        if any(tests):
            return False

        return not any([
            both_set_and_different(self.format, other.format),
            both_set_and_different(self.created, other.created),
            both_set_and_different(self.accessibility, other.accessibility),
        ])

    def __ne__(self, other):
        return not self.__eq__(other)

    def to_xml(self):
        """
        Convert self to XML, which can be send to API to register new digital
        instance.

        Returns:
            str: UTF-8 encoded string with XML representation.

        Raises:
            AssertionError: If :attr:`url` or :attr:`digital_library_id` is \
                            not set.
        """
        assert self.url, "You have to set .url!"
        assert self.digital_library_id, "You have to set .digital_library_id!"

        root = odict[
            "digitalInstance": odict[
                "@xmlns": "http://resolver.nkp.cz/v3/",
                "url": self.url,
                "digitalLibraryId": self.digital_library_id,
                "format": self.format,
                "accessibility": self.accessibility,
            ]
        ]

        if not self.format:
            del root["digitalInstance"]["format"]
        if not self.accessibility:
            del root["digitalInstance"]["accessibility"]

        return xmltodict.unparse(root, pretty=True).encode("utf-8")

    @staticmethod
    def instance_from_xmldict(dict_tag):
        """
        Create DigitalInstance from nested dicts (result of xmltodict).

        Args:
            dict_tag (dict): Nested dicts.

        Returns:
            obj: :class:`DigitalInstance` object.
        """
        return DigitalInstance(
            uid=dict_tag["@id"],
            active=dict_tag["@active"].lower() == "true",
            url=dict_tag.get("url", None),
            digital_library_id=dict_tag["digitalLibraryId"],
            format=dict_tag.get("format", None),
            created=dict_tag.get("created", None),
            deactivated=dict_tag.get("deactivated", None),
            accessibility=dict_tag.get("accessibility", None),
        )

    @staticmethod
    def from_xml(xml):
        """
        Parse `xml` string and DigitalInstances.

        Args:
            xml (str): Unicode/utf-8 XML.

        Returns:
            list: List of :class:`DigitalInstance` objects.
        """
        xdom = xmltodict.parse(xml)

        di = xdom["response"]["digitalInstances"]

        if not di.get("digitalInstance", False):
            return []

        return [
            DigitalInstance.instance_from_xmldict(dig_instance_tag)
            for dig_instance_tag in di["digitalInstance"]
        ]



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
"""
This module contains composers to allow creation of XML for URN:NBN, so you
don't have to create XML by hand.

See:
    - http://resolver.nkp.cz/api/v3/digDocRegistration.xsd
    - https://code.google.com/p/czidlo/wiki/ApiV3
"""
# Imports =====================================================================
import xmltodict

from odictliteral import odict
from kwargs_obj import KwargsObj


# Functions & classes =========================================================
class MonographComposer(KwargsObj):
    """
    Compostition class for Monograph publications.

    Attributes:
        title (str, required):  Title of the publication.
        subtitle (str): Subtitle of the publication.
        ccnb (str): CCNB number.
        isbn (str): ISBN string. You should validate this first.
        other_id (str): Useful for UUID and so on..
        document_type (str): Electronic? Scan?
        digital_born (bool): Was the publication digitally born, or is it scan?
        author (str): Author of the publication.
        publisher (str): Publishers name.
        place (str): Place where the publication was published (usually city).
        year (str): Year when the publication was published.
        format (str, required): PDF? EPUB?
    """
    def __init__(self, **kwargs):
        self.title = None
        self.subtitle = None
        self.ccnb = None
        self.isbn = None
        self.other_id = None
        self.document_type = None
        self.digital_born = True
        self.author = None
        self.publisher = None
        self.place = None
        self.year = None
        self.format = None

        self._all_set = True
        self._kwargs_to_attributes(kwargs)

    @staticmethod
    def _create_path(root, dict_type, path):
        """
        Create nested dicts in `root` using `dict_type` as constructor. Strings
        in `path` are used to construct the keys for nested dicts.

        Args:
            root (dict instance): Root dictionary, where the nested dicts will
                 be created.
            dict_type (dict-like class): Class which will be used to construct
                      dicts - ``dict_type()`.
            path (list/tuple of strings): List of keys for nested
                 ``dict_type``.

        Returns:
            dict: Return last nested dict (dict at last element at `path`).
        """
        for sub_path in path:
            if not isinstance(root.get(sub_path, None), dict):
                root[sub_path] = dict_type()

            root = root[sub_path]

        return root

    @staticmethod
    def _assign_pattern(where, key, what):
        """
        If `what`, assign `what` into ``where[key]``.

        Args:
            where (dict): Dith onto which the assingment action is performed.
            key (str): Definition of key for `where`.
            what (any): Any value. If evaluated to true, assign it to `where`.
        """
        if what:
            where[key] = what

    def _check_required_fields(self):
        """
        Make sure that all required fields are present.
        """
        assert self.title
        assert self.format

    def _add_identifier(self, mono_root, identifier, out=None):
        """
        Look for `identifier` in `self`. If found, add it to `mono_root` as
        `r:identifier`, or `r:out` if `out` is set. If not found, do nothing.

        Args:
            mono_root (dict): Dict into which you wish to add the `identifier`.
            identifier (str): Name of the attribute in `self`.
            out (str, default None): Optional argument to specify name of the
                key into which the `identifier` will be put.
        """
        out = out if out is not None else identifier

        if hasattr(self, identifier) and getattr(self, identifier) is not None:
            mono_root["r:" + out] = getattr(self, identifier)

    def to_xml_dict(self):
        """
        Compose hierarchical structure from ordered dicts, which will hold the
        XML.

        Returns:
            odict: Structure from ordered dicts.
        """
        self._check_required_fields()

        root = odict[
            "r:import": odict[
                "@xmlns:r": "http://resolver.nkp.cz/v3/",
                "r:monograph": odict[
                    "r:titleInfo": odict[
                        "r:title": self.title,
                    ],
                ],
            ]
        ]
        mono_root = root["r:import"]["r:monograph"]

        # subtitle
        MonographComposer._assign_pattern(
            mono_root["r:titleInfo"],
            "r:subTitle",
            self.subtitle
        )

        # handle ccnb, isbn, uuid
        self._add_identifier(mono_root, "ccnb")
        self._add_identifier(mono_root, "isbn")
        self._add_identifier(mono_root, "other_id", out="otherId")

        # add form of the book
        MonographComposer._assign_pattern(
            mono_root,
            "r:documentType",
            self.document_type
        )

        mono_root["r:digitalBorn"] = "true" if self.digital_born else "false"

        if self.author:
            mono_root["r:primaryOriginator"] = odict[
                "@type": "AUTHOR",
                "#text": self.author
            ]

        if any([self.place, self.publisher, self.year]):
            publ = odict()

            MonographComposer._assign_pattern(
                publ,
                "r:publisher",
                self.publisher
            )
            MonographComposer._assign_pattern(publ, "r:place", self.place)
            MonographComposer._assign_pattern(publ, "r:year", self.year)

            mono_root["r:publication"] = publ

        if self.format:
            format_dict = MonographComposer._create_path(
                root,
                odict,
                [
                    "r:import",
                    "r:digitalDocument",
                    "r:technicalMetadata",
                    "r:format",
                ]
            )

            format_dict["#text"] = self.format

        return root

    def to_xml(self):
        """
        Convert itself to XML string.

        Returns:
            str: XML.
        """
        return xmltodict.unparse(
            self.to_xml_dict(),
            pretty=True
        ).encode("utf-8")

    def __str__(self):
        return self.to_xml()


class MultiMonoComposer(MonographComposer):
    """
    Composition class for Multi monograph XMLs for URN:NBN.

    Attributes:
        title (str, required):  Title of the publication.
        volume_title (str, required): Title of the whole volume.
        subtitle (str): Subtitle of the publication.
        ccnb (str): CCNB number.
        isbn (str): ISBN string. You should validate this first.
        other_id (str): Useful for UUID and so on..
        document_type (str): Electronic? Scan?
        digital_born (bool): Was the publication digitally born, or is it scan?
        author (str): Author of the publication.
        publisher (str): Publishers name.
        place (str): Place where the publication was published (usually city).
        year (str): Year when the publication was published.
        format (str, required): PDF? EPUB?
    """
    def __init__(self, **kwargs):
        self.volume_title = None

        super(MultiMonoComposer, self).__init__()
        self._kwargs_to_attributes(kwargs)

    @staticmethod
    def _swap_keys(ordered_dict, old_key, new_key):
        """
        Swap `old_key` for `new_key`, but keep in mind position of key in
        `ordered_dict`.

        Returns:
            odict: Ordered dict with swapped keys.
        """
        return odict(
            (new_key, val) if key == old_key else (key, val)
            for key, val in ordered_dict.iteritems()
        )

    def _check_required_fields(self):
        """
        Make sure that all required fields are present.
        """
        assert self.volume_title
        super(MultiMonoComposer, self)._check_required_fields()

    def to_xml_dict(self):
        """
        Convert itself to XML string.

        Returns:
            str: XML.
        """
        root = super(MultiMonoComposer, self).to_xml_dict()

        # same as mono, except that <r:monograph> is now <r:monographVolume>
        root["r:import"] = MultiMonoComposer._swap_keys(
            root["r:import"],
            "r:monograph",
            "r:monographVolume"
        )

        # same as mono, except that <r:title> is now <r:monographTitle>
        mono_volume = root["r:import"]["r:monographVolume"]
        mono_volume["r:titleInfo"] = MultiMonoComposer._swap_keys(
            mono_volume["r:titleInfo"],
            "r:title",
            "r:monographTitle"
        )

        title_info = root["r:import"]["r:monographVolume"]["r:titleInfo"]
        if self.volume_title:
            title_info["r:volumeTitle"] = self.volume_title

        return root



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from collections import namedtuple


# Functions & classes =========================================================
class Catalog(namedtuple("Catalog", ["uid",
                                     "name",
                                     "created",
                                     "url_prefix"])):
    """
    Class used for representing informations about Catalogs, where the URN:NBN
    points.

    Attributes:
        uid (str): ID of the catalog.
        name (str): Name of the catalog.
        created (str): ISO 8601 date string .
        url_prefix (str): ..
    """



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from __future__ import unicode_literals

import pytest

from cz_urnnbn_api.api_structures import DigitalInstance

from ..test_xml_composer import data_context


# Fixtures ====================================================================
@pytest.fixture
def dig_inst_xml():
    return data_context("digital_instances.xml")


@pytest.fixture
def no_dig_inst_xml():
    return data_context("no_digital_instance.xml")


# Tests =======================================================================
def test_from_xml(dig_inst_xml):
    res = DigitalInstance.from_xml(dig_inst_xml)

    assert res
    assert len(res) == 3

    assert res[0] == DigitalInstance(
        uid="33",
        active=True,
        url="http://kramerius.mzk.cz/search/handle/uuid:8ffd7a5b-82da-11e0-bc9f-0050569d679d",
        digital_library_id="37",
        created="2012-09-03T00:44:34.603+02:00",
    )

    assert res[1] == DigitalInstance(
        uid="34",
        active=True,
        url="http://kramerius.mzk.cz/search/handle/uuid:8ffd7a5b-82da-11e0-bc9f-0050569d679d",
        digital_library_id="38",
        format="jpg;pdf",
        created="2012-09-19T00:37:25.362+02:00",
        accessibility="volně přístupné",
    )

    assert res[2] == DigitalInstance(
        uid="35",
        active=False,
        url="http://kramerius.mzk.cz/search/handle/uuid:8ffd7a5b-82da-11e0-bc9f-0050569d679d",
        digital_library_id="12",
        format="jpg;pdf",
        created="2012-09-19T00:39:41.117+02:00",
        deactivated="2012-09-19T00:42:30.334+02:00",
        accessibility="volně přístupné",
    )

    # test that access to non-existent attributes is dissabled
    with pytest.raises(ValueError):
        res[2].asd = 1


def test_from_xml_no_dig_instance(no_dig_inst_xml):
    res = DigitalInstance.from_xml(no_dig_inst_xml)

    assert res == []


def test_to_xml():
    di = DigitalInstance(
        url="http://kramerius3.mzk.cz/kramerius/handle/BOA001/935239",
        digital_library_id="4",
        format="jpg;pdf",
        accessibility="volně přístupné",
    )

    assert di.to_xml() == """<?xml version="1.0" encoding="utf-8"?>
<digitalInstance xmlns="http://resolver.nkp.cz/v3/">
\t<url>http://kramerius3.mzk.cz/kramerius/handle/BOA001/935239</url>
\t<digitalLibraryId>4</digitalLibraryId>
\t<format>jpg;pdf</format>
\t<accessibility>volně přístupné</accessibility>
</digitalInstance>""".encode("utf-8")


def test_to_xml_only_required_arguments():
    di = DigitalInstance(
        url="http://kramerius3.mzk.cz/kramerius/handle/BOA001/935239",
        digital_library_id="4",
    )

    assert di.to_xml() == """<?xml version="1.0" encoding="utf-8"?>
<digitalInstance xmlns="http://resolver.nkp.cz/v3/">
\t<url>http://kramerius3.mzk.cz/kramerius/handle/BOA001/935239</url>
\t<digitalLibraryId>4</digitalLibraryId>
</digitalInstance>""".encode("utf-8")


def test_to_xml_required_arguments_are_really_required():
    with pytest.raises(AssertionError):
        DigitalInstance(url="4", digital_library_id=None).to_xml()

    with pytest.raises(AssertionError):
        DigitalInstance(url=None, digital_library_id="4").to_xml()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import pytest
import faker  # fake-factory module

from cz_urnnbn_api import api
from cz_urnnbn_api import settings
from cz_urnnbn_api import MonographComposer


# Variables ===================================================================
# Fixtures ====================================================================
@pytest.fixture
def reg_code():
    return "edep"


# Tests =======================================================================
def test_is_valid_reg_code(reg_code):
    assert api.is_valid_reg_code(reg_code)


def test_iter_registrars(reg_code):
    registrars = list(api.iter_registrars())

    assert registrars

    assert any(reg for reg in registrars if reg.code == reg_code)


def test_get_registrar_info(reg_code):
    reg_info = api.get_registrar_info(reg_code)

    assert reg_info
    assert reg_info.code == reg_code
    assert reg_info.name == "Edeposit"
    assert not reg_info.description
    assert reg_info.modes == api.Modes(True, True, True)


def test_register_document_obj():
    fake = faker.Factory.create('cz_CZ')

    urn_nbn = api.register_document_obj(
        MonographComposer(
            title=fake.sentence(nb_words=3),
            author=fake.name(),
            format="pdf"
        )
    )

    assert str(urn_nbn)
    assert urn_nbn == api.get_urn_nbn_info(urn_nbn)

    assert api.get_digital_instances(urn_nbn) == []



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from urlparse import urljoin

import requests
import xmltodict
import dhtmlparser

import settings
from xml_composer import MonographComposer

from api_structures import *  # this is here to simplify working with API

# those are actually used in this module
from api_structures import URN_NBN
from api_structures import Registrar
from api_structures import DigitalInstance


# Functions & classes =========================================================
def _send_request(method, url, data=None, params=None):
    """
    Send request with data to server.

    Args:
        method (str): HTTP method - GET/POST/..
        url (str): URL of the resource where you wish to send request.
        data (dict, default None): This will be sent in body of the request.
        params (dict, default None): This may be sent as parameters.

    Raises:
        ValueError: In case that ``<error>`` tag was detected in output.
        requests.HTTPError: In case of HTTP error.

    Returns:
        str: Data returned from server.
    """
    resp = requests.request(
        method,
        url,
        data=data,
        params=params,
        verify=False,  # TODO: remove?
        headers={'Content-type': 'application/xml'},
        auth=(settings.USERNAME, settings.PASSWORD)
    )

    # handle http errors
    try:
        if resp.status_code not in [200, 400]:
            resp.raise_for_status()
    except requests.HTTPError, e:
        raise requests.HTTPError(e.message + "\n" + resp.content)

    # handle API errors
    dom = dhtmlparser.parseString(resp.content)
    if dom.find("error"):
        errors = [
            _get_content_or_str(error.find("code")) + " " +
            _get_content_or_str(error.find("message"))
            for error in dom.find("error")
        ]

        raise ValueError("\n".join(errors))

    return resp.text


def _get_content_or_str(tag, alt="", pick=lambda x: x[0]):
    """
    Get content of the `tag`, or return alternative defined by `alt`.

    In case that tag is instance of ``list`` or ``tuple``, `pick` function is
    applied.

    Args:
        tag (HTMLElement/list): Instance of HTMLElement or list.
        alt (obj, default ""): Alternative value for tag ``if not tag``.
        pick (fn, default lambda): Pick item from list of `tag`. Default lambda
             that picks first item.

    Return:
        str: Content of `tag`.
    """
    if not tag:
        return alt

    if isinstance(tag, list) or isinstance(tag, tuple):
        tag = pick(tag)

    return tag.getContent()


# API =========================================================================
def is_valid_reg_code(reg_code=settings.REG_CODE):
    """
    Check whether `reg_code` is valid registrar code.

    Args:
        reg_code (str): Producent's registration code.

    Returns:
        bool: True, if the `reg_code` is valid.
    """
    try:
        return _send_request(
            method="GET",
            url=urljoin(settings.REG_URL, reg_code),
        )
    except (requests.exceptions.HTTPError, ValueError):
        return False

    return True


def iter_registrars():
    """
    Iterate over all registrars.

    Yields:
        obj: :class:`.Registrar` instance with basic informations.
    """
    data = _send_request(
        method="GET",
        url=urljoin(settings.URL, "registrars")
    )

    xdom = xmltodict.parse(data)

    for registrar_tag in xdom["response"]["registrars"]["registrar"]:
        yield Registrar.from_xmldict(registrar_tag)


def get_registrar_info(reg_code):
    """
    Get detailed informations about registrar with `reg_code`.

    Args:
        reg_code (str): Code identifier of registrar.

    Returns:
        obj: :class:`.Registrar` instance with all informations.
    """
    data = _send_request(
        method="GET",
        url=urljoin(settings.REG_URL, reg_code)
    )

    xdom = xmltodict.parse(data)
    reg_tag = xdom["response"]["registrar"]

    return Registrar.from_xmldict(reg_tag)


def register_document_obj(xml_composer, reg_code=settings.REG_CODE):
    """
    Register document in mode ``BY_RESOLVER`` - let the resolver give you
    URN:NBN code.

    Args:
        xml_composer (obj): Instance of the :class:`MonographComposer`.
        reg_code (str, default settings.REG_CODE): Registrar's code.

    Returns:
        obj: Instance of :class:`.URN_NBN` which contains assinged URN:NBN \
             code.
    """
    msg = "`xml_composer` parameter have to by subclass of MonographComposer!"
    assert isinstance(xml_composer, MonographComposer), msg

    return register_document(xml_composer.to_xml(), reg_code)


def register_document(xml, reg_code=settings.REG_CODE):
    """
    Register document in mode ``BY_RESOLVER`` - let the resolver give you
    URN:NBN code.

    Args:
        xml (str): XML, which will be used for registration. See
                   :mod:`xml_composer` for details.
        reg_code (str, default settings.REG_CODE): Registrar's code.

    Returns:
        obj: Instance of :class:`.URN_NBN` which contains assinged URN:NBN \
             code.
    """
    result = _send_request(
        method="POST",
        url=urljoin(settings.URL, "registrars/%s/digitalDocuments") % reg_code,
        data=xml
    )

    return URN_NBN.from_xmldict(
        xmltodict.parse(result)
    )


def register_digital_instance_obj(urn_nbn, digital_instance):
    """
    Register `digital_instance` object as new digital instance of the
    document for given `urn_nbn`

    Args:
        urn_nbn (str): URN:NBN identifier of registered document.
        digital_instance (obj): :class:`DigitalInstance` instance.

    Returns:
        obj: :class:`DigitalInstance` with more informations.
    """
    assert isinstance(digital_instance, DigitalInstance)

    result = _send_request(
        method="POST",
        url=urljoin(settings.URL, "resolver/%s/digitalInstances") % urn_nbn,
        data=digital_instance.to_xml()
    )

    return DigitalInstance.from_xml(result)[0]


def register_digital_instance(urn_nbn, url, digital_library_id, format=None,
                              accessibility=None):
    """
    Compose and register new digital instance of document.

    Args:
        urn_nbn (str): URN:NBN identifier of registered document.
        url (str): URL of the digital instance.
        digital_library_id (str): ID of the digital library.
        format (str, def. None): Format of the instance - ``pdf`` for example.
        accessibility (str, def. None): Is the registration neccessary to \
                      access this instance?

    Returns:
        obj: :class:`DigitalInstance` with more informations.
    """
    di = DigitalInstance(
        url=url,
        digital_library_id=digital_library_id,
        format=format,
        accessibility=accessibility
    )

    return register_digital_instance_obj(urn_nbn, di)


def get_digital_instances(urn_nbn):
    """
    Get list of :class:`.DigitalInstance` objects for given `urn_nbn`.

    DigitalInstances are `pointers` to :class:`.DigitalLibrary`, where the
    instance of document is stored. There should be always a link to the
    document in ``url`` property.

    Returns:
        list: :class:`.DigitalInstance` objects or blank list.
    """
    result = _send_request(
        method="GET",
        url=urljoin(settings.URL, "resolver/%s/digitalInstances") % urn_nbn
    )

    return DigitalInstance.from_xml(result)


def get_urn_nbn_info(urn_nbn):
    """
    For given `urn_nbn` string, return parsed :class:`URN_NBN` object.

    Args:
        urn_nbn (str): String.

    Return:
        obj: :class:`URN_NBN` object with additional info in properties.
    """
    result = _send_request(
        method="GET",
        url=urljoin(settings.URL, "urnnbn/%s") % urn_nbn
    )

    return URN_NBN.from_xmldict(
        xmltodict.parse(result)
    )


def get_full_urn_nbn_record(urn_nbn):
    """
    Return full record for given `urn_nbn`.

    Warning:
        This function doesn't yet return ORM data, just plain string.

    Args:
        urn_nbn (str): String.

    Returns:
        str: XML with full informations about URN:NBN record.
    """
    nbn_url = "resolver/%s?action=show&format=xml" % urn_nbn
    result = _send_request(
        method="GET",
        url=urljoin(settings.URL, nbn_url)
    )

    return result  # TODO: create ORM for the XML



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import os

import pytest

from cz_urnnbn_api.xml_convertor import convert_mono_xml
from cz_urnnbn_api.xml_convertor import convert_mono_volume_xml

from test_xml_composer import data_context
from test_xml_composer import mono_out_example
from test_xml_composer import multimono_out_example


# Fixtures ====================================================================
@pytest.fixture
def mono_mods_example():
    return data_context("mods_mono.xml")


# Tests =======================================================================
def test_mono_xml_conversion(mono_mods_example, mono_out_example):
    out = convert_mono_xml(mono_mods_example, "pdf")

    assert out == mono_out_example


def test_mono_volume_xml_conversion(mono_mods_example, multimono_out_example):
    out = convert_mono_volume_xml(mono_mods_example, "pdf")

    assert out == multimono_out_example



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from setuptools import setup, find_packages

from docs import getVersion


# Variables ===================================================================
CHANGELOG = open('CHANGELOG.rst').read()
LONG_DESCRIPTION = "\n\n".join([
    open('README.rst').read(),
    open('CONTRIBUTORS.rst').read(),
    CHANGELOG
])
URL = "https://resolver.nkp.cz"


# Actual setup definition =====================================================
setup(
    name='cz-urnnbn-api',
    version=getVersion(CHANGELOG),
    description="API for the Czech URN:NBN resolver (%s)." % URL,
    long_description=LONG_DESCRIPTION,
    url='https://github.com/edeposit/cz-urnnbn-api/',

    author='Edeposit team',
    author_email='edeposit@email.cz',

    classifiers=[
        "Development Status :: 3 - Alpha",

        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",

        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications :: File Sharing",

        "License :: OSI Approved :: MIT License",
    ],
    license='MIT',

    packages=find_packages('src'),
    package_dir={'': 'src'},

    include_package_data=True,
    zip_safe=False,

    install_requires=open("requirements.txt").read().splitlines(),
    extras_require={
        "test": [
            "pytest",
            "fake-factory"
        ],
        "docs": [
            "sphinx",
            "sphinxcontrib-napoleon",
        ]
    }
)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from modes import Modes
from catalog import Catalog
from urn_nbn import URN_NBN
from registrar import Registrar
from digital_library import DigitalLibrary
from digital_instance import DigitalInstance
